import java.util.*;

public class Queens2SAT {

	private static boolean DEBUG = false;

	private static int N = -1;
	private static StringBuffer out = new StringBuffer();
	
	private static final String VARS_PLACEHOLDER = "VARS";
	private static final String CLAUSES_PLACEHOLDER = "CLAUSES";
	public static final String VERSION = "v. 20070505";

	private static int vars = 0, clauses = 0;
	private static void endclause() {
		if (out.charAt( out.length()-1 ) != ' ') out.append(" ");
		out.append("0\n");
		clauses++;
	}
	
	public static void main(String args[]) {
		System.err.println("This is Queens2SAT, " + VERSION);
		if (args.length == 0) {
			System.err.println("This programs prints on standard output a SAT instance encoding the N-queens problem");
			System.err.println("\nSyntax: java Queens2SAT [-d] n");
			System.err.println("  -d:  prints output in readable format, with many comments"); System.err.println("       (otherwise output will be in standard DIMACS format)");
			System.err.println("   n:  the number of queens, to be placed on a n x n board");			
			System.exit(1);
		}
		
		// Reads command-line arguments
		for (int i=0; i < args.length; i++) {
			String arg = args[i];
			if (arg.equals("-d")) DEBUG = true;
			// else if( another option....)....
			else {
				try {
					N = Integer.parseInt(arg);
				} catch(NumberFormatException ex) {
					// ignored
					System.err.println("Argument " + arg + " unknown: ignored");
				}
			}
		}
		if (N <= 0) {
			System.err.println("Invalid argument: argument N is mandatory and must be positive.");
			System.exit(2);
		}
		buildCNF();
		
		String outString = out.toString();
		outString = outString.replaceAll(VARS_PLACEHOLDER, ""+vars);
		outString = outString.replaceAll(CLAUSES_PLACEHOLDER, ""+clauses);		
		
		System.out.println(outString);
	}
	
	private static class Coord implements Cloneable {
		private int x;
		private int y;
		protected Coord() {}
		protected Coord(int xx, int yy) { x=xx; y=yy; }
		protected String toDimacsVar() {  
			if (!DEBUG) return ""+((x-1)*N + y);
			else return "q("+x+","+y+")";
		}
		protected Coord(Coord base, int deltaX, int deltaY) {
			this.x = base.x + deltaX;
			this.y = base.y + deltaY;
		}
		protected Coord(String dimacsVar) {
			int i = Integer.parseInt(dimacsVar);
			this.y = ( (i-1) % N )  +  1;
			this.x = 1 + (i - this.y) / N;
		}
		protected void move(int deltaX, int deltaY) {
			this.x += deltaX;
			this.y += deltaY;
		}
		protected boolean inBounds() {
			return x >= 1 && x <= N && y >= 1 && y <= N;
		}
		public Object clone() {
			try {
				return super.clone();
			} catch(CloneNotSupportedException ex) {
				throw new InternalError();
			  }
		}
		public boolean equals(Object o) {
			if (o == null) return false;
			if (!o.getClass().equals(this.getClass())) return false;
			Coord oo = (Coord)o;
			return x == oo.x && y == oo.y;
		}
	} // end inner class Coord
	
	
	private static void buildCNF() {
		header();
		clauses();
	}
	
	private static void header() {
		out.append("c This CNF has been created by Queens2SAT " + VERSION + " on " + new Date() + "\n");
		if (DEBUG) out.append("c DEBUG MODE: output is NOT in DIMACS format!\n");
		out.append("c N = " + N + "\n");
		out.append("c START DICTIONARY\n");
		for (int x=1; x<=N; x++) {
			for (int y=1; y<=N; y++) {
				String q = new Coord(x,y).toDimacsVar();
				vars++; // counts the total number of vars
				out.append("c queen("+x+","+y+"): " + q+"\n");
			}
		}
		out.append("c END DICTIONARY\n");
		out.append("p cnf "+VARS_PLACEHOLDER + " " + CLAUSES_PLACEHOLDER + "\n");
	}
	
	private static void clauses() {
		oneQueenPerRow();
		oneQueenPerColumn();
		oneQueenPerDiag();
	}
	
	
	private static void oneQueenPerRow() {
		for(int x=1; x<=N; x++) {
			debug("At least one queen in row " + x);		
			for(int y=1; y<=N; y++) {
				out.append( new Coord(x,y).toDimacsVar() + " ");
			}
			endclause();
			
			debug("At most one queen in row " + x);			
			for(int y1=1; y1<=N; y1++) {
				for(int y2=y1+1; y2<=N; y2++) {
					// q(x,y1) --> -q(x,y2)
					out.append( "-"+new Coord(x,y1).toDimacsVar() + " -" + new Coord(x, y2).toDimacsVar() );
					endclause();
				}
			}
		}
	}

	private static void oneQueenPerColumn() {
		for(int y=1; y<=N; y++) {
			debug("At least one queen in column " + y);		
			for(int x=1; x<=N; x++) {
				out.append( new Coord(x,y).toDimacsVar() + " ");
			}
			endclause();
			
			debug("At most one queen in column " + y);
			for(int x1=1; x1<=N; x1++) {
				for(int x2=x1+1; x2<=N; x2++) {
					// q(x1,y) --> -q(x2,y)
					out.append( "-"+new Coord(x1,y).toDimacsVar() + " -" + new Coord(x2,y).toDimacsVar() );
					endclause();
				}
			}
		}
	}

	private static void oneQueenPerDiag() {
		for(int y=1; y<=N-1; y++) {  // upper horiz edge
			Coord q1 = new Coord(1, y);

			debug("No two queens on the diagonal starting from (1,"+y+") and going SW");
			while(q1.inBounds()) {
				Coord q2 = (Coord)q1.clone();  
				q2.move(+1, -1);  // going SW				
				while (q2.inBounds()) {
					out.append( "-"+q1.toDimacsVar() + " -" + q2.toDimacsVar()); 
					endclause();
					q2.move(+1, -1);
				}
				q1.move(+1, -1);	
			}

			q1 = new Coord(1, y);
			debug("No two queens on the diagonal starting from (1,"+y+") and going SE");
			while(q1.inBounds()) {
				Coord q2 = (Coord)q1.clone();  
				q2.move(+1, +1);  // going SE				
				while (q2.inBounds()) {
					out.append( "-"+q1.toDimacsVar() + " -" + q2.toDimacsVar()); 
					endclause();
					q2.move(+1, +1);
				}
				q1.move(+1, +1);	
			}
		}
		
		
		
		for(int y=1; y<=N-1; y++) {  // lower horiz edge
			Coord q1 = new Coord(N, y);

			debug("No two queens on the diagonal starting from ("+N+","+y+") and going NW");
			while(q1.inBounds()) {
				Coord q2 = (Coord)q1.clone();  
				q2.move(-1, -1);  // going NW				
				while (q2.inBounds()) {
					out.append( "-"+q1.toDimacsVar() + " -" + q2.toDimacsVar()); 
					endclause();
					q2.move(-1, -1);
				}
				q1.move(-1, -1);	
			}

			q1 = new Coord(N, y);
			debug("No two queens on the diagonal starting from ("+N+","+y+") and going NE");
			while(q1.inBounds()) {
				Coord q2 = (Coord)q1.clone();  
				q2.move(-1, +1);  // going NE				
				while (q2.inBounds()) {
					out.append( "-"+q1.toDimacsVar() + " -" + q2.toDimacsVar()); 
					endclause();
					q2.move(-1, +1);
				}
				q1.move(-1, +1);	
			}
		}
		
	}
			
	private static void debug(String msg) {
		if (DEBUG) out.append("c " + msg + "\n");
	}
} //:~	